import 'dart:math';

import 'package:flutter/material.dart';
import 'package:audio_session/audio_session.dart';
import 'package:flutter/material.dart';
import 'package:just_audio/just_audio.dart';
class ChatVoiceMessageWidget extends StatefulWidget {
  final String url;
  final bool isLocal;

  const ChatVoiceMessageWidget({
    Key key,
    @required this.url,
    this.isLocal = false,
  }) : super(key: key);

  @override
  _ChatVoiceMessageWidgetState createState() => _ChatVoiceMessageWidgetState();
}

class _ChatVoiceMessageWidgetState extends State<ChatVoiceMessageWidget>
    with WidgetsBindingObserver {
  AudioPlayer _audioPlayer;
  bool _isPlaying;

  @override
  initState() {
    super.initState();
    WidgetsBinding.instance?.addObserver(this);

    _audioPlayer = AudioPlayer();

    _isPlaying = false;

    _init();
  }

  Future<void> _init() async {
    // Inform the operating system of our app's audio attributes etc.
    // We pick a reasonable default for an app that plays speech.
    final session = await AudioSession.instance;
    await session.configure(AudioSessionConfiguration.speech());

    // Listen to errors during playback.
    _audioPlayer.playbackEventStream.listen((event) {
      if (event?.processingState == ProcessingState.completed) {
        _audioPlayer.seek(Duration.zero);
        _isPlaying = false;
        setState(() {});
      }
    }, onError: (Object e, StackTrace stackTrace) {
      print('A stream error occurred: $e');
    });

    // Try to load audio from a source and catch any errors.
    try {
      await _audioPlayer.setUrl(widget.url, preload: true);
    } catch (e) {
      print("Error loading audio source: $e");
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance?.removeObserver(this);
    // Release decoders and buffers back to the operating system making them
    // available for other apps to use.
    _audioPlayer.dispose();
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      // Release the player's resources when not in use. We use "stop" so that
      // if the app resumes later, it will still remember what position to
      // resume from.
      _audioPlayer.stop();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: Colors.blue,
        borderRadius: BorderRadius.circular(8),
      ),
      child: Row(
        children: [
          IconButton(
            onPressed: () => _playVoiceMessage(),
            padding: const EdgeInsets.all(0),
            iconSize: 32,
            icon: Icon(
              _isPlaying ? Icons.pause_rounded : Icons.play_arrow_rounded,
              color: Colors.white,
            ),
          ),
          Expanded(
            child: SliderTheme(
              data: SliderTheme.of(context).copyWith(
                activeTrackColor: Colors.white,
                inactiveTrackColor: Color(0xff6F717F),
                thumbColor: Colors.indigo,
                overlayColor: Colors.blue,
                thumbShape: RoundSliderThumbShape(enabledThumbRadius: 8.0),
                overlayShape: RoundSliderOverlayShape(overlayRadius: 12.0),
              ),
              child: VoiceSliderWidget(_audioPlayer),
            ),
          ),
          // VoiceDurationTextWidget(_audioPlayer),
          // SizedBox(width: 5),
        ],
      ),
    );
  }

  void _playVoiceMessage() {
    if (_isPlaying) {
      _audioPlayer.pause();
      _isPlaying = false;
    } else {
      _audioPlayer.play();
      _isPlaying = true;
    }

    setState(() {});
  }
}

class VoiceSliderWidget extends StatelessWidget {
  final AudioPlayer audioPlayer;

  const VoiceSliderWidget(this.audioPlayer, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<PlaybackEvent>(
        stream: audioPlayer.playbackEventStream,
        builder: (context, snapshot) {
          if (snapshot.hasData && snapshot.data != null) {
            return Slider(
              value: min(
                  (snapshot.data.bufferedPosition ?? Duration.zero)
                      .inMilliseconds
                      .toDouble(),
                  (snapshot.data.duration ?? Duration.zero)
                      .inMilliseconds
                      .toDouble()),
              max: (snapshot.data.duration ?? Duration.zero)
                  .inMilliseconds
                  .toDouble(),
              min: 0.0,
              onChanged: (t) =>
                  audioPlayer.seek(Duration(milliseconds: t.round())),
            );
          } else {
            return SizedBox();
          }
        });
  }
}

class VoiceDurationTextWidget extends StatelessWidget {
  final AudioPlayer audioPlayer;

  const VoiceDurationTextWidget(this.audioPlayer, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<Duration>(
        stream: audioPlayer.durationStream,
        builder: (context, durationSnapshot) {
          if (durationSnapshot.hasData && durationSnapshot.data != null) {
            return StreamBuilder<Duration>(
                stream: audioPlayer.bufferedPositionStream,
                builder: (context, positionSnapshot) {
                  if (positionSnapshot.hasData &&
                      positionSnapshot.data != null) {
                    Duration duration = Duration(
                      microseconds:
                      (durationSnapshot.data?.inMilliseconds ?? 0) -
                          (positionSnapshot.data?.inMilliseconds ?? 0),
                    );

                    return Text(
                      duration.inHours.toString().padLeft(2, '0') +
                          ":" +
                          duration.inMinutes
                              .remainder(60)
                              .toString()
                              .padLeft(2, '0') +
                          ":" +
                          duration.inSeconds
                              .remainder(60)
                              .toString()
                              .padLeft(2, '0'),
                    );
                  } else {
                    return SizedBox();
                  }
                });
          } else {
            return SizedBox();
          }
        });
  }
}
